@@ -1,4 +1,4 @@
-"""Markdown to pofiles extractor according to mdpo specification."""
+"""Markdown to PO files extractor according to mdpo specification."""
 
 import os
 
@@ -28,10 +28,12 @@ class Md2Po:
         'ignore_msgids',
         'command_aliases',
         'mark_not_found_as_obsolete',
+        'preserve_not_found',
         'extensions',
         'plaintext',
         'include_codeblocks',
         'metadata',
+        'events',
 
         '_current_msgid',
         '_current_tcomment',
@@ -41,6 +43,7 @@ class Md2Po:
         '_disable_next_line',
         '_enable_next_line',
         '_include_next_codeblock',
+        '_disable_next_codeblock',
 
         '_enterspan_replacer',
         '_leavespan_replacer',
@@ -111,11 +114,15 @@ class Md2Po:
         self.mark_not_found_as_obsolete = kwargs.get(
             'mark_not_found_as_obsolete', True,
         )
+        self.preserve_not_found = kwargs.get('preserve_not_found', True)
 
         self.extensions = kwargs.get(
             'extensions',
             DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
         )
+        self.events = {}
+        if 'events' in kwargs:
+            self.events.update(kwargs['events'])
 
         self.plaintext = kwargs.get('plaintext', False)
 
@@ -126,6 +133,7 @@ class Md2Po:
         self._enable_next_line = False
 
         self._include_next_codeblock = False
+        self._disable_next_codeblock = False
 
         self.metadata = {}
 
@@ -359,6 +367,62 @@ class Md2Po:
         self._current_tcomment = None
         self._current_msgctxt = None
 
+    def command(self, mdpo_command, comment, original_command):
+        # raise 'command' event
+        try:
+            pre_event = self.events['command']
+        except KeyError:
+            pass
+        else:
+            if pre_event(
+                self,
+                mdpo_command,
+                comment,
+                original_command,
+            ) is False:
+                return
+
+        if mdpo_command == 'mdpo-disable-next-line':
+            self._disable_next_line = True
+        elif mdpo_command == 'mdpo-disable':
+            self._disable = True
+        elif mdpo_command == 'mdpo-enable':
+            self._disable = False
+        elif mdpo_command == 'mdpo-enable-next-line':
+            self._enable_next_line = True
+        elif mdpo_command == 'mdpo-include-codeblock':
+            self._include_next_codeblock = True
+        elif mdpo_command == 'mdpo-disable-codeblock':
+            self._disable_next_codeblock = True
+        elif mdpo_command == 'mdpo-disable-codeblocks':
+            self.include_codeblocks = False
+        elif mdpo_command == 'mdpo-include-codeblocks':
+            self.include_codeblocks = True
+        elif mdpo_command == 'mdpo-translator':
+            if not comment:
+                raise ValueError(
+                    'You need to specify a string for the'
+                    ' extracted comment with the command'
+                    f' \'{original_command}\'.',
+                )
+            self._current_tcomment = comment.rstrip()
+        elif mdpo_command == 'mdpo-context':
+            if not comment:
+                raise ValueError(
+                    'You need to specify a string for the'
+                    f' context with the command \'{original_command}\'.',
+                )
+            self._current_msgctxt = comment.rstrip()
+        elif mdpo_command == 'mdpo-include':
+            if not comment:
+                raise ValueError(
+                    'You need to specify a message for the'
+                    ' comment to include with the command'
+                    f' \'{original_command}\'.',
+                )
+            self._current_msgid = comment.rstrip()
+            self._save_current_msgid()
+
     def _process_command(self, text):
         original_command, comment = parse_mdpo_html_command(text)
         if original_command is None:
@@ -369,43 +433,21 @@ class Md2Po:
         except KeyError:  # not custom command
             command = original_command
 
-        if command == 'mdpo-disable-next-line':
-            self._disable_next_line = True
-        elif command == 'mdpo-disable':
-            self._disable = True
-        elif command == 'mdpo-enable':
-            self._disable = False
-        elif command == 'mdpo-enable-next-line':
-            self._enable_next_line = True
-        elif command == 'mdpo-translator':
-            if not comment:
-                raise ValueError(
-                    'You need to specify a string for the'
-                    ' extracted comment with the command'
-                    f' \'{original_command}\'.',
-                )
-            self._current_tcomment = comment.rstrip()
-        elif command == 'mdpo-context':
-            if not comment:
-                raise ValueError(
-                    'You need to specify a string for the'
-                    f' context with the command \'{original_command}\'.',
-                )
-            self._current_msgctxt = comment.rstrip()
-        elif command == 'mdpo-include-codeblock':
-            self._include_next_codeblock = True
-        elif command == 'mdpo-include':
-            if not comment:
-                raise ValueError(
-                    'You need to specify a message for the'
-                    ' comment to include with the command'
-                    f' \'{original_command}\'.',
-                )
-            self._current_msgid = comment.rstrip()
-            self._save_current_msgid()
+        # process solved command
+        self.command(command, comment, original_command)
 
     def enter_block(self, block, details):
         # print("ENTER BLOCK:", block.name)
+
+        # raise 'enter_block' event
+        try:
+            pre_event = self.events['enter_block']
+        except KeyError:
+            pass
+        else:
+            if pre_event(self, block, details) is False:
+                return
+
         if block.value == md4c.BlockType.P:
             self._inside_pblock = True
         elif block.value == md4c.BlockType.CODE:
@@ -417,16 +459,31 @@ class Md2Po:
             if self._uls_deep > 1:
                 # changing UL deeep
                 self._save_current_msgid()
+        elif block.value == md4c.BlockType.QUOTE:
+            if self._inside_liblock:
+                self._save_current_msgid()
         elif block.value == md4c.BlockType.HTML:
             self._inside_htmlblock = True
 
     def leave_block(self, block, details):
         # print("LEAVE BLOCK:", block.name)
+
+        # raise 'leave_block' event
+        try:
+            pre_event = self.events['leave_block']
+        except KeyError:
+            pass
+        else:
+            if pre_event(self, block, details) is False:
+                return
+
         if block.value == md4c.BlockType.CODE:
             self._inside_codeblock = False
-            if self._include_next_codeblock or self.include_codeblocks:
-                self._save_current_msgid()
-                self._include_next_codeblock = False
+            if not self._disable_next_codeblock:
+                if self.include_codeblocks or self._include_next_codeblock:
+                    self._save_current_msgid()
+            self._include_next_codeblock = False
+            self._disable_next_codeblock = False
         elif block.value == md4c.BlockType.HTML:
             self._inside_htmlblock = False
         else:
@@ -438,8 +495,18 @@ class Md2Po:
                 self._uls_deep -= 1
             self._save_current_msgid()
 
-    def enter_span(self, span, details, *args):
+    def enter_span(self, span, details):
         # print("ENTER SPAN:", span.name, details)
+
+        # raise 'enter_span' event
+        try:
+            pre_event = self.events['enter_span']
+        except KeyError:
+            pass
+        else:
+            if pre_event(self, span, details) is False:
+                return
+
         if not self.plaintext:
             # underline spans for double '_' character enters two times
             if not self._inside_uspan:
@@ -474,6 +541,16 @@ class Md2Po:
 
     def leave_span(self, span, details):
         # print("LEAVE SPAN:", span.name)
+
+        # raise 'leave_span' event
+        try:
+            pre_event = self.events['leave_span']
+        except KeyError:
+            pass
+        else:
+            if pre_event(self, span, details) is False:
+                return
+
         if not self.plaintext:
             if not self._inside_uspan:
                 if span.value == md4c.SpanType.WIKILINK:
@@ -517,7 +594,17 @@ class Md2Po:
                 self._inside_uspan = False
 
     def text(self, block, text):
-        # print("TEXT:", text)
+        # print(f"TEXT: '{text}'")
+
+        # raise 'text' event
+        try:
+            pre_event = self.events['text']
+        except KeyError:
+            pass
+        else:
+            if pre_event(self, block, text) is False:
+                return
+
         if not self._inside_htmlblock:
             if not self._inside_codeblock:
                 if self._inside_liblock and text == '\n':
@@ -558,8 +645,9 @@ class Md2Po:
                     return
                 self._current_msgid += text
             else:
-                if self._include_next_codeblock or self.include_codeblocks:
-                    self._current_msgid += text
+                if not self._disable_next_codeblock:
+                    if self.include_codeblocks or self._include_next_codeblock:
+                        self._current_msgid += text
         else:
             self._process_command(text)
 
@@ -614,7 +702,9 @@ class Md2Po:
             for entry in self.pofile:
                 if entry not in self.found_entries:
                     _equal_not_obsolete_found = find_entry_in_entries(
-                        entry, self.found_entries, compare_obsolete=False,
+                        entry,
+                        self.found_entries,
+                        compare_obsolete=False,
                     )
                     if _equal_not_obsolete_found:
                         self.pofile.remove(entry)
@@ -622,16 +712,16 @@ class Md2Po:
                         entry.obsolete = True
                 else:
                     entry.obsolete = False
-        else:
+        elif not self.preserve_not_found:
             for entry in self.pofile:
                 if entry not in self.found_entries:
                     _equal_not_obsolete_found = find_entry_in_entries(
-                        entry, self.found_entries, compare_obsolete=False,
+                        entry,
+                        self.found_entries,
+                        compare_obsolete=False,
                     )
-                    if _equal_not_obsolete_found:
+                    if not _equal_not_obsolete_found:
                         self.pofile.remove(entry)
-                    else:
-                        entry.obsolete = False
 
         if self.metadata:
             self.pofile.metadata.update(self.metadata)
@@ -653,6 +743,7 @@ def markdown_to_pofile(
     plaintext=False,
     wrapwidth=78,
     mark_not_found_as_obsolete=True,
+    preserve_not_found=True,
     extensions=DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
     po_encoding=None,
     md_encoding='utf-8',
@@ -661,6 +752,7 @@ def markdown_to_pofile(
     ignore_msgids=[],
     command_aliases={},
     metadata={},
+    events={},
     **kwargs,
 ):
     """Extracts all the msgids from a string of Markdown content or a group of
@@ -698,6 +790,9 @@ def markdown_to_pofile(
         mark_not_found_as_obsolete (bool): The strings extracted from markdown
             that will not be found inside the provided pofile will be marked
             as obsolete.
+        preserve_not_found (bool): The strings extracted from markdown that
+            will not be found inside the provided pofile wouldn't be removed.
+            Only has effect if ``mark_not_found_as_obsolete`` is ``False``.
         extensions (list): md4c extensions used to parse markdown content,
             formatted as a list of 'pymd4c' keyword arguments. You can see all
             available at `pymd4c repository <https://github.com/dominickpastore
@@ -718,7 +813,24 @@ def markdown_to_pofile(
             instead of ``<!-- mdpo-enable -->``, you can pass the dictionaries
             ``{"mdpo-on": "mdpo-enable"}`` or ``{"mdpo-on": "enable"}`` to this
             parameter.
-        metadata (dict): Metadata to include in the produced PO file.
+        metadata (dict): Metadata to include in the produced PO file. If the
+            file contains previous metadata fields, these will be updated
+            preserving the values of the already defined.
+        events (dict): MD4C preprocessing events executed during the parsing
+            process. You can use these to customize the extraction process.
+            Takes functions are values. If one of these functions returns
+            ``False``, the block parsing is skipped by md2po. The available
+            events are:
+
+            * ``enter_block``: Executed when the parsing a Markdown block
+              starts.
+            * ``leave_block``: Executed when the parsing a Markdown block ends.
+            * ``enter_span``: Executed when the parsing of a Markdown span
+              starts.
+            * ``leave_span``: Executed when the parsing of a Markdown span
+              ends.
+            * ``text``: Executed when the parsing of text starts ends.
+            * ``command``: Executed when a mdpo HTML command is found.
 
     Examples:
         >>> content = 'Some text with `inline code`'
@@ -742,12 +854,14 @@ def markdown_to_pofile(
         plaintext=plaintext,
         wrapwidth=wrapwidth,
         mark_not_found_as_obsolete=mark_not_found_as_obsolete,
+        preserve_not_found=preserve_not_found,
         extensions=extensions,
         xheaders=xheaders,
         include_codeblocks=include_codeblocks,
         ignore_msgids=ignore_msgids,
         command_aliases=command_aliases,
         metadata=metadata,
+        events=events,
         **kwargs,
     ).extract(
         po_filepath=po_filepath,
