@@ -18,7 +18,7 @@ from mdpo.md4c import DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS
 
 DESCRIPTION = (
     'Utility like xgettext to extract Markdown content dumping it'
-    ' inside pofiles.'
+    ' inside PO files.'
 )
 
 
@@ -33,7 +33,7 @@ def build_parser():
     )
     parser.add_argument(
         '-i', '--ignore', dest='ignore', default=[],
-        help='Filepaths to ignore when ``GLOB_OR_CONTENT`` argument'
+        help='Filepaths to ignore when \'GLOB_OR_CONTENT\' argument'
              ' is a glob, as a list of comma separated values.',
         metavar='PATH_1,PATH_2...',
     )
@@ -41,14 +41,15 @@ def build_parser():
         '-po', '--po-filepath', dest='po_filepath',
         default=None,
         help='Merge new msgids in the po file indicated at this parameter (if'
-             ' ``--save`` argument is passed) or use the msgids of the file as'
-             ' reference for ``--mark-not-found-as-obsolete`` parameter.',
+             ' \'--save\' argument is passed) or use the msgids of the file as'
+             ' reference for mark not found as obsoletes if'
+             ' \'--merge-pofiles\' parameter is not passed.',
         metavar='OUTPUT_PO_FILEPATH',
     )
     parser.add_argument(
         '-s', '--save', dest='save', action='store_true',
         help='Save new found msgids to the po file'
-             ' indicated as parameter ``--filepath``.',
+             ' indicated as parameter \'--filepath\'.',
     )
     parser.add_argument(
         '-mo', '--mo-filepath', dest='mo_filepath',
@@ -66,18 +67,25 @@ def build_parser():
     )
     parser.add_argument(
         '-w', '--wrapwidth', dest='wrapwidth',
-        help='Wrap width for po file indicated at ``--filepath`` parameter.'
-             ' Only useful when the ``-w`` option was passed to xgettext.',
+        help='Wrap width for po file indicated at \'--filepath\' parameter.'
+             ' Only useful when the \'-w\' option was passed to xgettext.',
         metavar='N', type=int,
     )
     parser.add_argument(
         '-m', '--merge-pofiles',
         dest='mark_not_found_as_obsolete',
         action='store_false',
-        help='New found msgids not present in the pofile passed at '
-             '``--po-filepath`` parameter will be preserved in the new pofile,'
-             ' even when are not been found in the current extraction. If this'
-             ' argument is not passed, will be marked as obsolete strings.',
+        help='Messages not found which are already stored in the PO file'
+             ' passed as \'--po-filepath\' argument will not be marked as'
+             ' obsolete.',
+    )
+    parser.add_argument(
+        '-r', '--remove-not-found',
+        dest='preserve_not_found',
+        action='store_false',
+        help='Messages not found which are already stored in the PO file'
+             ' passed as \'--po-filepath\' parameter will be removed.'
+             ' Only has effect used in combination with \'--merge-pofiles\'.',
     )
     parser.add_argument(
         '-x', '--extensions', dest='extensions',
@@ -85,7 +93,7 @@ def build_parser():
             DEFAULT_MD4C_GENERIC_PARSER_EXTENSIONS,
         ),
         help='md4c extensions used to parse markdown content, separated by'
-             ' ``,`` and formatted as pymd4c extension keyword arguments.'
+             ' \',\' and formatted as pymd4c extension keyword arguments.'
              ' You can see all available at https://github.com/dominickpastore'
              '/pymd4c#parser-option-flags',
     )
@@ -101,7 +109,7 @@ def build_parser():
         '-a', '--xheaders', dest='xheaders',
         action='store_true',
         help='Include mdpo specification x-headers. These only will be'
-             ' included if you do not pass the parameter ``--plaintext``.',
+             ' included if you do not pass the parameter \'--plaintext\'.',
     )
     parser.add_argument(
         '-c', '--include-codeblocks',
@@ -121,6 +129,8 @@ def build_parser():
         metavar='Key:Value',
         help='Custom metadata key-value pairs to include in the produced'
              ' PO file. This argument can be passed multiple times.'
+             ' If the file contains previous metadata fields, these will'
+             ' be updated preserving the values of the already defined.'
              ' For example, to define utf-8 encoding and Spanish language use'
              ' -d "Content-Type: text/plain; charset=utf-8" -d "Language: es"',
     )
@@ -168,6 +178,7 @@ def run(args=[]):
         mo_filepath=opts.mo_filepath,
         plaintext=opts.plaintext,
         mark_not_found_as_obsolete=opts.mark_not_found_as_obsolete,
+        preserve_not_found=opts.preserve_not_found,
         extensions=opts.extensions,
         po_encoding=opts.po_encoding,
         md_encoding=opts.md_encoding,
