@@ -1,4 +1,4 @@
-"""Markdown files translator using pofiles as reference."""
+"""Markdown files translator using PO files as reference."""
 
 import glob
 import re
@@ -72,6 +72,7 @@ class Po2Md:
         '_inside_indented_codeblock',
         '_inside_pblock',
         '_inside_liblock',
+        '_inside_liblock_first_p',
         '_inside_codespan',
         '_codespan_start_index',
         '_codespan_backticks',
@@ -179,6 +180,8 @@ class Po2Md:
 
         self._inside_pblock = False
         self._inside_liblock = False
+        # first li block paragraph (li block "title")
+        self._inside_liblock_first_p = False
 
         self._inside_codespan = False
         self._codespan_start_index = None
@@ -374,21 +377,23 @@ class Po2Md:
                         self._save_current_line()
                 self._ol_marks[-1][0] += 1
                 self._current_line += '{}1{} '.format(
-                    '   ' * (len(self._ol_marks) - 1),
+                    '   ' * (len(self._current_list_type) - 1),
                     self._ol_marks[-1][1],
                 )
                 self._current_list_type[-1][-1].append(False)
             else:
                 # inside UL
                 self._current_line += '{}{} '.format(
-                    '   ' * (len(self._ul_marks) - 1), self._ul_marks[-1],
+                    '   ' * (len(self._current_list_type) - 1),
+                    self._ul_marks[-1],
                 )
                 if details['is_task']:
                     self._current_line += '[%s] ' % details['task_mark']
                 self._current_list_type[-1][-1].append(details['is_task'])
             self._inside_liblock = True
+            self._inside_liblock_first_p = True
         elif block.value == md4c.BlockType.UL:
-            if len(self._ul_marks) > 0:
+            if self._current_list_type:
                 self._save_current_msgid()
                 self._save_current_line()
             self._current_list_type.append(['ul', []])
@@ -397,15 +402,20 @@ class Po2Md:
             self._current_list_type.append(['ol', []])
             self._ol_marks.append([0, details['mark_delimiter']])
         elif block.value == md4c.BlockType.HR:
-            self._current_line += '---'
             if not self._inside_liblock:
-                self._current_line += '\n\n'
+                self._current_line += '---\n\n'
+            else:
+                # inside lists, the separator '---' can't be used
+                self._current_line += '***'
         elif block.value == md4c.BlockType.TH:
             self._current_line += '| '
             self._current_thead_aligns.append(details['align'].value)
         elif block.value == md4c.BlockType.TD:
             self._current_line += '| '
         elif block.value == md4c.BlockType.QUOTE:
+            if self._inside_liblock:
+                self._save_current_msgid()
+                self._save_current_line()
             self._inside_quoteblock = True
         elif block.value == md4c.BlockType.HTML:
             self._inside_htmlblock = True
@@ -415,12 +425,31 @@ class Po2Md:
 
         if block.value == md4c.BlockType.P:
             self._save_current_msgid()
-            if not self._inside_liblock:
+
+            if self._inside_liblock:
+                if self._inside_quoteblock:
+                    self._current_line = '{}{}'.format(
+                        '   ' * len(self._current_list_type),
+                        self._current_line,
+                    )
+                    self._save_current_line()
+                else:
+                    if self._inside_liblock_first_p:
+                        self._inside_liblock_first_p = False
+                    else:
+                        self._current_line = '\n{}{}'.format(
+                            '   ' * len(self._current_list_type),
+                            self._current_line,
+                        )
+                        self._save_current_line()
+            else:
                 self._save_current_line()
+
             self._inside_pblock = False
             if self._inside_quoteblock:
                 self._current_line = '>'
                 self._save_current_line()
+
         elif block.value == md4c.BlockType.CODE:
             self._save_current_msgid()
             self._inside_codeblock = False
@@ -508,7 +537,9 @@ class Po2Md:
         elif block.value == md4c.BlockType.QUOTE:
             if self._outputlines[-1] == '>':
                 self._outputlines.pop()
-            self._save_current_line()
+            if not self._inside_liblock:
+                self._save_current_line()
+            self._inside_quoteblock = False
         elif block.value == md4c.BlockType.TABLE:
             if not self._inside_quoteblock:
                 self._save_current_line()
@@ -690,7 +721,7 @@ def pofile_to_markdown(
     wrapwidth=80,
     **kwargs,
 ):
-    r"""Translate Markdown content or a file using pofiles for message replacing.
+    r"""Translate Markdown content or a file using PO files for message replacing.
 
     This implementation reproduces the same valid Markdown output, given the
     provided AST, with replaced translations, but doesn't rebuilds the same
